
package org.lejos.ev3.conveyorbelt;

import lejos.hardware.Brick;
import lejos.hardware.BrickFinder;
import lejos.hardware.Button;
import lejos.hardware.lcd.TextLCD;
import lejos.hardware.motor.NXTRegulatedMotor;
import lejos.hardware.motor.UnregulatedMotor;
import lejos.hardware.port.SensorPort;
import lejos.hardware.sensor.EV3TouchSensor;
import lejos.hardware.sensor.EV3UltrasonicSensor;
import lejos.hardware.sensor.SensorMode;
import lejos.robotics.SampleProvider;
import lejos.utility.Delay;

/**
 * @author 		Jarema Czajkowski
 * @see			ev3instructions.blogspot.com
 * @created 	19-05-2017
 * @license 	GNU General Public License
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 * 
 * 
 */

class KeyListenerThread extends Thread {
	
	public KeyListenerThread() {
		
	}
	
	public void run() {
		for(;;) {
        	int buttoncode = Button.readButtons();
			if (buttoncode == Button.ID_ESCAPE) {
		        System.exit(1); 
		        return;
			}
		}
	}
}	

public class ConveyorBelt {

	private static TextLCD EV3LCD;
	private static int queue;
	private static final int queueMax = 3;
	private static final double maxDistance = 0.05;
	
	public static void main(String[] args) {
        Brick EV3Brick = BrickFinder.getDefault();
        EV3LCD = EV3Brick.getTextLCD();
		drawMessage("Initializing...");
		
		/**
		 * It supports the direction of movement of the conveyor belt
		 */  
		NXTRegulatedMotor myAMotor = new NXTRegulatedMotor(EV3Brick.getPort("A"));
		myAMotor.setSpeed(100);
        
        /**
         * Moves the frame in which the object is
         */
        UnregulatedMotor myBMotor = new UnregulatedMotor(EV3Brick.getPort("B"));
        
		/**
         * Get an instance of the Touch EV3 sensor
         */
        EV3TouchSensor myS2Sensor = null;
		EV3TouchSensor myS3Sensor = null;
        try {
        	myS2Sensor = new EV3TouchSensor(SensorPort.S2);
        	myS3Sensor = new EV3TouchSensor(SensorPort.S3);
        } catch (Exception ex) {
        	System.exit(ex.hashCode());
        }
        
        /**
         * Get an instance of this sensor in touch mode.
         * 
         * The sample contains one element, a value of 
         *   0 indicates that the button is not pressed
         *   1 indicates the button is pressed 
         */
        SensorMode myS2Provider = myS2Sensor.getTouchMode();
        SensorMode myS3Provider = myS3Sensor.getTouchMode();
        
        /**
         * Get an instance of the Ultrasonic EV3 sensor
         */
        EV3UltrasonicSensor myUSensor = null;
        SampleProvider myUProvider = null;
        try {
        	myUSensor = new EV3UltrasonicSensor(SensorPort.S1);
        	myUProvider = myUSensor.getDistanceMode();
        } catch (Exception ex) {
        	System.exit(ex.hashCode());
        }
        
        /**
         *  Start separate thread to manage actions assigned 
         *  to physical buttons  
         */
    	drawMessage("Start listener...");
        KeyListenerThread myThread = new KeyListenerThread();
        myThread.start(); 
        
        /**
         *  Initialize an array of floats for fetching samples. 
         *  Ask the SampleProvider how long the array should be
         */
        float[] sample = new float[myS2Provider.sampleSize()];
       
        /**
         *  Proper work begins here
         */
        	drawMessage("Transporting...");
        	Button.LEDPattern(3);
	        myAMotor.forward();
	        
	        queue = 1; 
	        double value;
			do {
	        	myUProvider.fetchSample(sample, 0);
	        	value = Double.valueOf(sample[0]);
	        	
//	        	System.out.println(String.format( "%.3f %b %b", value ,Double.isInfinite(value) ,value>maxDistance ));
	        	Delay.msDelay(100);
	        } while (value > maxDistance);
	        myAMotor.stop();
	        
	        drawMessage("Processing...");
	        myBMotor.setPower(80);
	        myBMotor.forward();
	        
	        Button.LEDPattern(5);
	        do {
	           myS3Provider.fetchSample(sample, 0);
	           if (Math.round(sample[0]) == 1) { myBMotor.stop(); myBMotor.backward(); queue++; }
	           
	           myS2Provider.fetchSample(sample, 0);
	           if (Math.round(sample[0]) == 1) { myBMotor.forward(); }
	           
	           drawMessage("Queue "+String.valueOf(queue));
	           Delay.msDelay(1000);
	        }
	        while (queue <= queueMax);  
	        myBMotor.stop();
	        myBMotor.setPower(0);
	        
	        drawMessage("Waiting...");
	        Button.LEDPattern(3);    
	        myAMotor.rotate(360);	        
	}

	protected static void drawMessage(String str) {
        EV3LCD.clear(1);
        EV3LCD.drawString(str, 1, 1);      
	}

}
